@@ -30,10 +30,17 @@ impl<T> SendSync for T {}
 
 /// Immutable reference-counted container.
 #[cfg(not(feature = "sync"))]
-pub use std::rc::Rc as Shared;
+// TODO: Further audit no_std compatibility
+// When building with no_std + sync features, explicit imports from alloc
+// are needed despite using no_std_compat. This fixed compilation errors
+// around missing trait implementations for some users.
+pub use alloc::rc::Rc as Shared;
 /// Immutable reference-counted container.
 #[cfg(feature = "sync")]
-pub use std::sync::Arc as Shared;
+// TODO: Further audit no_std compatibility
+// While no_std_compat should map std::sync::Arc to alloc::sync::Arc,
+// there appear to be cases where this mapping fails.
+pub use alloc::sync::Arc as Shared;
 
 /// Synchronized shared object.
 #[cfg(not(feature = "sync"))]
@@ -305,7 +312,7 @@ impl<'a> NativeCallContext<'a> {
                     ERR::ErrorMismatchOutputType(
                         cast_type.into(),
                         result_type.into(),
-                        Position::NONE,
+                        self.position(),
                     )
                     .into()
                 })
@@ -338,7 +345,7 @@ impl<'a> NativeCallContext<'a> {
                     ERR::ErrorMismatchOutputType(
                         cast_type.into(),
                         result_type.into(),
-                        Position::NONE,
+                        self.position(),
                     )
                     .into()
                 })
@@ -438,7 +445,7 @@ impl<'a> NativeCallContext<'a> {
                     args,
                     is_ref_mut,
                     false,
-                    Position::NONE,
+                    self.position(),
                 )
                 .map(|(r, ..)| r);
         }
@@ -467,7 +474,7 @@ impl<'a> NativeCallContext<'a> {
                 args,
                 is_ref_mut,
                 is_method_call,
-                Position::NONE,
+                self.position(),
             )
             .map(|(r, ..)| r)
     }
