@@ -93,7 +93,11 @@
 #![allow(clippy::no_effect_underscore_binding)] // Underscored variables may be used by code within feature guards
 #![allow(clippy::semicolon_if_nothing_returned)] // One-liner `match` cases are sometimes formatted as multi-line blocks
 
-#[cfg(feature = "no_std")]
+// TODO: Further audit no_std compatibility
+// When building with no_std + sync features, explicit imports from alloc
+// are needed despite using no_std_compat. This fixed compilation errors
+// in `native.rs` around missing trait implementations for some users.
+//#[cfg(feature = "no_std")]
 extern crate alloc;
 
 #[cfg(feature = "no_std")]
@@ -328,6 +332,9 @@ pub use optimizer::OptimizationLevel;
 #[cfg(feature = "internals")]
 pub use types::dynamic::{AccessMode, DynamicReadLock, DynamicWriteLock, Variant};
 
+#[cfg(feature = "internals")]
+pub use module::{FuncInfo, FuncMetadata};
+
 #[cfg(feature = "internals")]
 #[cfg(not(feature = "no_float"))]
 pub use types::FloatWrapper;
@@ -337,9 +344,9 @@ pub use types::{BloomFilterU64, CustomTypeInfo, Span, StringsInterner};
 
 #[cfg(feature = "internals")]
 pub use tokenizer::{
-    get_next_token, is_valid_function_name, is_valid_identifier, parse_string_literal, InputStream,
-    MultiInputsStream, Token, TokenIterator, TokenizeState, TokenizerControl,
-    TokenizerControlBlock,
+    get_next_token, is_valid_function_name, is_valid_identifier, parse_raw_string_literal,
+    parse_string_literal, InputStream, MultiInputsStream, Token, TokenIterator, TokenizeState,
+    TokenizerControl, TokenizerControlBlock,
 };
 
 #[cfg(feature = "internals")]
