@@ -69,7 +69,8 @@ impl FnNamespace {
     }
 }
 
-/// A type containing the metadata of a single registered function.
+/// _(internals)_ A type containing the metadata of a single registered function.
+/// Exported under the `internals` features only.
 #[derive(Debug, Clone, Eq, PartialEq, Hash)]
 #[non_exhaustive]
 pub struct FuncMetadata {
@@ -86,12 +87,15 @@ pub struct FuncMetadata {
     /// Parameter types (if applicable).
     pub param_types: FnArgsVec<TypeId>,
     /// Parameter names and types (if available).
+    /// Exported under the `metadata` feature only.
     #[cfg(feature = "metadata")]
     pub params_info: FnArgsVec<Identifier>,
     /// Return type name.
+    /// Exported under the `metadata` feature only.
     #[cfg(feature = "metadata")]
     pub return_type: Identifier,
     /// Comments.
+    /// Exported under the `metadata` feature only.
     #[cfg(feature = "metadata")]
     pub comments: crate::StaticVec<SmartString>,
 }
@@ -152,6 +156,25 @@ impl FuncMetadata {
     }
 }
 
+/// Information about a function, native or scripted.
+///
+/// Exported under the `internals` feature only.
+#[allow(dead_code)]
+pub struct FuncInfo<'a> {
+    /// Function metadata.
+    pub metadata: &'a FuncMetadata,
+    /// Function namespace.
+    ///
+    /// Not available under `no_module`.
+    #[cfg(not(feature = "no_module"))]
+    pub namespace: Identifier,
+    /// Metadata if the function is scripted.
+    ///
+    /// Not available under `no_function`.
+    #[cfg(not(feature = "no_function"))]
+    pub script: Option<crate::ScriptFnMetadata<'a>>,
+}
+
 /// _(internals)_ Calculate a [`u64`] hash key from a namespace-qualified function name and parameter types.
 /// Exported under the `internals` feature only.
 ///
@@ -400,7 +423,7 @@ impl FuncRegistration {
     {
         #[cfg(feature = "metadata")]
         {
-            // Do not update parameter informations if `with_params_info` was called previously.
+            // Do not update parameter information if `with_params_info` was called previously.
             if self.metadata.params_info.is_empty() {
                 let mut param_type_names = FUNC::param_names()
                     .iter()
@@ -2327,7 +2350,8 @@ impl Module {
         let _ = result?;
 
         // Encapsulated environment
-        let environ = Shared::new(crate::ast::EncapsulatedEnviron {
+        #[cfg(not(feature = "no_function"))]
+        let env = Shared::new(crate::ast::EncapsulatedEnviron {
             #[cfg(not(feature = "no_function"))]
             lib: ast.shared_lib().clone(),
             imports,
@@ -2340,7 +2364,7 @@ impl Module {
         while i > 0 {
             i -= 1;
 
-            let (mut value, mut aliases) = if i >= orig_scope_len {
+            let (mut _value, mut aliases) = if i >= orig_scope_len {
                 let (_, v, a) = scope.pop_entry().unwrap();
                 (v, a)
             } else {
@@ -2348,9 +2372,10 @@ impl Module {
                 (v.clone(), a.to_vec())
             };
 
-            value.deep_scan(|v| {
+            #[cfg(not(feature = "no_function"))]
+            _value.deep_scan(|v| {
                 if let Some(fn_ptr) = v.downcast_mut::<crate::FnPtr>() {
-                    fn_ptr.environ = Some(environ.clone());
+                    fn_ptr.env = Some(env.clone());
                 }
             });
 
@@ -2359,7 +2384,7 @@ impl Module {
                 1 => {
                     let alias = aliases.pop().unwrap();
                     if !module.contains_var(&alias) {
-                        module.set_var(alias, value);
+                        module.set_var(alias, _value);
                     }
                 }
                 _ => {
@@ -2373,12 +2398,12 @@ impl Module {
                         if first_alias.is_none() {
                             first_alias = Some(alias);
                         } else {
-                            module.set_var(alias, value.clone());
+                            module.set_var(alias, _value.clone());
                         }
                     }
 
                     if let Some(alias) = first_alias {
-                        module.set_var(alias, value);
+                        module.set_var(alias, _value);
                     }
                 }
             }
@@ -2393,15 +2418,11 @@ impl Module {
             })
             .for_each(|f| {
                 let hash = module.set_script_fn(f.clone());
-                if let (
-                    RhaiFunc::Script {
-                        environ: ref mut e, ..
-                    },
-                    _,
-                ) = module.functions.as_mut().unwrap().get_mut(&hash).unwrap()
+                if let (RhaiFunc::Script { env: ref mut e, .. }, _) =
+                    module.functions.as_mut().unwrap().get_mut(&hash).unwrap()
                 {
                     // Encapsulate AST environment
-                    *e = Some(environ.clone());
+                    *e = Some(env.clone());
                 }
             });
 
